''***************************************
''*  Easy Bluetooth  v0.1               *
''*  Author: Anthony Myatt              *
''*  Copyright (c) 2010 Anthony Myatt   *
''*  See end of file for terms of use.  *
''***************************************

{-----------------REVISION HISTORY-----------------
 v1.0 -                                            }

CON

' Define Delimiters
StartDelimiter                 = $02     ' "STX"
EndDelimiter                   = $03     ' "ETX"

' Define Packet Types
REQ_PACKET                     = $52     ' "R"
CFM_PACKET                     = $43     ' "C"
IND_PACKET                     = $69     ' "i"
RES_PACKET                     = $72     ' "r"

' Define OP Codes
GAP_INQUIRY                    = $00     ' Initiates a search for other Bluetooth devices
GAP_DEVICE_FOUND               = $01     ' Indicates that a device has been found
GAP_REMOTE_DEVICE_NAME         = $02     ' Request the user-friendly name from a known remote Bluetooth device
GAP_READ_LOCAL_NAME            = $03     ' Request the user-friendly name for the local Bluetooth device
GAP_WRITE_LOCAL_NAME           = $04     ' Change the user-friendly name for the local Bluetooth device
GAP_READ_LOCAL_BDA             = $05     ' Read the Bluetooth device address of the local Bluetooth device
GAP_SET_SCANMODE               = $06     ' Change the Bluetooth scan mode
GAP_GET_FIXED_PIN              = $16     ' Reads the current fixed pin code from NVS
GAP_SET_FIXED_PIN              = $17     ' Stores a new fixed pin code in NVS
GAP_GET_PIN                    = $75     ' Send a PIN code to the RBT-001 module as response to a GAP_GET_PIN indication
GAP_GET_SECURITY_MODE          = $18     ' 
GAP_SET_SECURITY_MODE          = $19
GAP_REMOVE_PAIRING             = $1B
GAP_LIST_PAIRED_DEVICES        = $1C
GAP_ENTER_SNIFF_MODE           = $21
GAP_EXIT_SNIFF_MODE            = $37
GAP_ENTER_PARK_MODE            = $38
GAP_EXIT_PARK_MODE             = $39
GAP_ENTER_HOLD_MODE            = $3A
GAP_SET_LINK_POLICY            = $3B
GAP_GET_LINK_POLICY            = $3C
GAP_POWER_SAVE_MODE_CHANGED    = $3D
GAP_ACL_ESTABLISHED            = $50
GAP_ACL_TERMINATED             = $51
SPP_SET_PORT_CONFIG            = $07
SPP_GET_PORT_CONFIG            = $08
SPP_PORT_CONFIG_CHANGED        = $09
SPP_ESTABLISH_LINK             = $0A
SPP_LINK_ESTABLISHED           = $0B
SPP_INCOMMING_LINK_ESTABLISHED = $0C
SPP_RELEASE_LINK               = $0D
SPP_LINK_RELEASED              = $0E
SPP_SEND_DATA                  = $0F
SPP_INCOMING_DATA              = $10
SPP_TRANSPARENT_MODE           = $11
SPP_CONNECT_DEFAULT_CON        = $12
SPP_STORE_DEFAULT_CON          = $13
SPP_GET_LIST_DEFAULT_CON       = $14
SPP_DELETE_DEFAULT_CON         = $15
SPP_SET_LINK_TIMEOUT           = $57
SPP_GET_LINK_TIMEOUT           = $58
SPP_PORT_STATUS_CHANGED        = $3E
SPP_GET_PORT_STATUS            = $40
SPP_PORT_SET_DTR               = $41
SPP_PORT_SET_RTS               = $42
SPP_PORT_BREAK                 = $43
SPP_PORT_OVERRUN_ERROR         = $44
SPP_PORT_PARITY_ERROR          = $45
SPP_PORT_FRAMING_ERROR         = $46
SDAP_CONNECT                   = $32
SDAP_DISCONNECT                = $33
SDAP_CONNECTION_LOST           = $34
SDAP_SERVICE_BROWSE            = $35
SDAP_SERVICE_SEARCH            = $36
SDAP_SERVICE_REQUEST           = $1E
SDAP_ATTRIBUTE_REQUEST         = $3F
CHANGE_NVS_UART_SPEED          = $23
CHANGE_UART_SETTINGS           = $48
SET_PORTS_TO_OPEN              = $22
GET_PORTS_TO_OPEN              = $1F
RESTORE_FACTORY_SETTINGS       = $1A
STORE_CLASS_OF_DEVICE          = $28
FORCE_MASTER_ROLE              = $1D
READ_OPERATION_MODE            = $49
WRITE_OPERATION_MODE           = $4A
SET_DEFAULT_LINK_POLICY        = $4C
GET_DEFAULT_LINK_POLICY        = $4D
SET_EVENT_FILTER               = $4E
GET_EVENT_FILTER               = $4F
SET_DEFAULT_LINK_TIMEOUT       = $55
GET_DEFAULT_LINK_TIMEOUT       = $56
SET_DEFAULT_LINK_LATENCY       = $63
GET_DEFAULT_LINK_LATENCY       = $64
SET_PCM_SLAVE_CONFIG           = $74
ENABLE_SDP_RECORD              = $29
DELETE_SDP_RECORDS             = $2A
STORE_SDP_RECORD               = $31
RESET                          = $26
READY                          = $25
TEST_MODE                      = $24
WRITE_ROM_PATCH                = $47
READ_RSSI                      = $20
RF_TEST_MODE                   = $4B
DISABLE_TL                     = $52
TL_ENABLED                     = $53
AWAIT_INITIALIZATION_EVENT     = $66
ENTER_BLUETOOTH_MODE           = $67
READ_NVS                       = $72
WRITE_NVS                      = $73

' Define other constants
MAX_BYTES_TO_RECEIVE           = 60

VAR

Byte dataSet[340], msgBuffer[255], read_data[255], cog
long stack[60]

Long Started
OBJ
                                                                                   
serial : "Full_Serial"
DB : "Full_Serial"

Pub Start (RXpin, TXPin, Baud) : okay | ptr

    Started := serial.start(RXpin, TXpin, 0, Baud) ' Start the serial driver
    DB.start(31, 30, 0, 9600)                      ' Start the serial driver on prop-plug pins for debug
    return started                                 ' return if the main serial driver started

PUB Stop

    serial.stop                                    ' Stop the main serial driver
    Started := false                               ' Set started to false

PRI calculate_crc                                  ' Calculate the checksum

  dataSet[5] := (dataSet[1] + dataSet[2] + dataSet[3] + dataSet[4]) & $ff

PRI receive_response(ms) | i, T, l

  i := 0
  
  repeat  
    T := read_data[i++] := serial.rxTime(ms)       ' Receive a byte
  while (T => 0 and i =< MAX_BYTES_TO_RECEIVE)     ' While Data is still there and the amount of bytes is not higher than MAX_BYTES_TO_RECEIVE

  repeat l from 0 to i - 2
    DB.tx(read_data[l])                            ' Output received packet to debug port
  
  if (((read_data[1] + read_data[2] + read_data[3] + read_data[4]) & $ff) == read_data[5]) ' Make sure the checksum is correct
    return i - 1
  else
    return -1
  
PRI send_cmd_receive_response(size, ms) | crc, i

  calculate_crc                                    ' Calculate the checksum
  repeat i from 0 to size                          ' For the size of the packet
    serial.tx(dataSet[i])                          ' Send a byte to the BT module
    'DB.tx(dataSet[i])
  'waitcnt(clkfreq * 3 / 10 + cnt)
  return receive_response(ms)                      ' Return the response

PRI sendREQ (opCode, msg, length, ms) | ptr, dataLength ' Send a REQ packet to the BT module

  if Started                                            ' If the serial driver has been started
    ptr := 0                                            ' Set the pointer to 0
    dataLength := length + 1                            ' The length of the data contained within the packet is one byte larger because it also contains the size of the data
    dataSet[ptr++] := StartDelimiter                    ' Start building the packet (first byte is $02)
    dataSet[ptr++] := REQ_PACKET                        ' Then the REQ packet type identifier
    dataSet[ptr++] := opCode                            ' Then the OP Code
    dataSet[ptr++] := dataLength                        ' The LSB of the data Length
    dataSet[ptr++] := dataLength >> 8                   ' The MSB of the data Length
    dataSet[ptr++] := $00                               ' Where the checksum will go (set to $00 for the moment)
    bytemove(@dataSet[ptr],msg,length + 1)              ' The actual data to be sent
    dataSet[ptr+length+1] := EndDelimiter               ' Finaly the end delimiter ($03)
    send_cmd_receive_response(length+7, ms)             ' Send the command and read in the response

PUB Set_Local_Name (name) | len, ptr, i                 ' Set the local name of the BT module

  len := strsize(name) + 1                              ' Set len to the length of the name plus one for the null terminator
  if 0 < len < 40                                       ' Make sure length is between 0 and 40
    msgBuffer[0] := len                                 ' Set the first byte to the length of the name
    Repeat ptr from 1 to len                            ' For each byte of the string
      msgBuffer[ptr] := byte[name][ptr - 1]             ' Fill msgBuffer with the byte
    sendREQ(GAP_WRITE_LOCAL_NAME, @msgBuffer, len, 100) ' Send a REQ packet with the OP code GAP_WRITE_LOCAL_NAME and allow 100ms for a response

PUB Send_Data (data) | len, ptr, i

  len := strsize(data) + 3
  if 0 < len < 40 
    msgBuffer[0] := $01
    msgBuffer[1] := (len - 2)
    msgBuffer[2] := (len - 2) >> 8
    Repeat ptr from 3 to len
      msgBuffer[ptr] := byte[data][ptr - 3]
    sendREQ(SPP_SEND_DATA, @msgBuffer, len, 100)

PUB Set_PIN (pin) | len, ptr, i

  len := strsize(pin)
  if 0 < len < 16 
    msgBuffer[0] := len
    Repeat ptr from 1 to len
      msgBuffer[ptr] := byte[pin][ptr - 1]
    sendREQ(GAP_SET_FIXED_PIN, @msgBuffer, len, 100)

PUB Establish_Link (address) | ptr

    msgBuffer[0] := $01
    Repeat ptr from 1 to 6
      msgBuffer[ptr] := byte[address][ptr - 1]
    msgBuffer[7] := $01
    sendREQ(SPP_ESTABLISH_LINK, @msgBuffer, 7, 2000)

PUB Get_Link_Timeout (address) | ptr

    Repeat ptr from 0 to 5
      msgBuffer[ptr] := byte[address][ptr]
    sendREQ(SPP_GET_LINK_TIMEOUT, @msgBuffer, 5, 100)

PUB Read_Real_RSSI

    msgBuffer[0] := $1A
    msgBuffer[1] := $4B
    msgBuffer[2] := $0E
    msgBuffer[3] := $00
    msgBuffer[4] := $06                 
    sendREQ($70, @msgBuffer, 4, 100)

PUB Enter_Transparent_Mode

    msgBuffer[0] := $01
    sendREQ(SPP_TRANSPARENT_MODE, @msgBuffer, 0, 100)

PUB Get_Security_Mode

    sendREQ(GAP_GET_SECURITY_MODE, @msgBuffer, -1, 100)

PUB Get_My_Address | len, ptr, i

    sendREQ(GAP_READ_LOCAL_BDA, @msgBuffer, -1, 100)

PUB Set_Operation_Mode (value) | len, ptr, i

    if value
      msgBuffer[0] := $01
    else
      msgBuffer[0] := $00

    sendREQ(WRITE_OPERATION_MODE, @msgBuffer, 0, 100)

PUB Soft_Reset | len, ptr, i

  sendREQ(RESET, $00, 0, 500)
    